# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/data_generator.ipynb (unless otherwise specified).

__all__ = ['MNISTDataGenerator']

# Cell
from fastcore.test import *
from nbdev.showdoc import *
import numpy as np
import jovsatools
import tensorflow as tf

# Cell

class MNISTDataGenerator(object):
    """Generates a multi-calss regression/classification data based on MNIST

        x = mnist data flattened
        y = regression/classification targets based on self.initialize()

        Arguments:
            additional_y: Integer, used to specify the additional targets
            seed = Integer, used to specify np.random.seed()

        Returns:
            train_x: numpy.ndarray, MNIST train data flattened
            train_y: numpy.ndarray, all the targets; MNIST label + additional_y
            test_x: numpy.ndarray, MNIST test data flattened
            test_y: numpy.ndarray, all the targets; MNIST label + additional_y
    """
    def __init__(self, additional_y=0, seed=1123):
        self.additional_y = additional_y
        self.seed = np.random.seed(seed=seed)

        # needed to store additional generation functions
        self._func_map = {}
        self._initialize(additional_y)

        assert list(filter(lambda x: x is None, [
            self.mnist_train_x,
            self.mnist_train_y,
            self.mnist_test_x,
            self.mnist_test_y
        ])) == []

        self.train_n = len(self.mnist_train_x)
        self.test_n = len(self.mnist_test_x)

    def _initialize(self, additional_y):
        """Prepare functions to approximate """
        epsilon = 0.000123
        C = 102 # emperical value from analyzing MNIST data

        # label 0: mnist class
        if additional_y >= 1:
            # label 1: classification (0, 1) target
            self._func_map[0] = lambda x: int(np.random.random()<0.9) if 2*(x[10]**3)-2*x[3]+15 > 8.9 else 0

        if additional_y >= 2:
            # label 2: unbounded regression target
            self._func_map[1] = lambda x: np.log(np.sum(x)+epsilon)

        if additional_y >= 3:
            # label 3: bounded (0, 1) regression target
            self._func_map[2] = lambda x: np.mean(x)/C

        # regular MNIST train/test data
        train, test = tf.keras.datasets.mnist.load_data()
        self.mnist_train_x, self.mnist_train_y = train[0], train[1]
        self.mnist_test_x, self.mnist_test_y = test[0], test[1]

    def prepare_datasets(self, mnist_x, mnist_y, n):
        """Main worker function for dataset preparation """
        x, y = [], []
        for i in range(n):
            features = mnist_x[i].flatten().reshape(-1, 1)
            labels = [mnist_y[i]]
            for f in range(self.additional_y):
                labels.append(self._func_map[f](features))
            x.append(features)
            y.append(labels)
        return np.asarray(x), np.asarray(y)

    def __call__(self, sample_n):
        """Data generation call """
        assert sample_n <= self.train_n, f"max alloable size is {self.train_n}"
        train_x, train_y = [], []
        test_x, test_y = [], []

        train_x, train_y = self.prepare_datasets(self.mnist_train_x, self.mnist_train_y, n=sample_n)
        test_x, test_y = self.prepare_datasets(self.mnist_test_x, self.mnist_test_y, n=self.test_n)
        return (train_x, train_y, test_x, test_y)


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/sample_mesa_model.ipynb (unless otherwise specified).

__all__ = ['MoneyAgent', 'MoneyModel']

# Cell
from nbdev.showdoc import *
import jovsatools
import fastcore
import matplotlib.pyplot as plt
from mesa import Agent, Model
from mesa.time import RandomActivation
import numpy as np

# Cell
class MoneyAgent(Agent):
    """ An agent with fixed initial wealth."""
    def __init__(self, unique_id, model):
        super().__init__(unique_id, model)
        self.wealth = 1

    def step(self):
        if self.wealth == 0:
            return
        other_agent = self.random.choice(self.model.schedule.agents)
        other_agent.wealth += 1
        self.wealth -= 1

class MoneyModel(Model):
    """A model with some number of agents."""
    def __init__(self, N):
        self.num_agents = N
        self.schedule = RandomActivation(self)
        # Create agents
        for i in range(self.num_agents):
            a = MoneyAgent(i, self)
            self.schedule.add(a)

    def step(self):
        '''Advance the model by one step.'''
        self.schedule.step()


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/custom_pytorch_training.ipynb (unless otherwise specified).

__all__ = ['generate_1d_data', 'generate_2d_data', 'train', 'eval_loss', 'train_epochs', 'Histogram',
           'MixtureOfLogistics']

# Cell

import numpy as np
import matplotlib.pyplot as plt
import torch
import torch.nn as nn
import torch.nn.functional as F
import torch.optim as optim
import torch.utils.data as data

from nbdev.export import *

# Cell
def generate_1d_data(n, d):
    rand = np.random.RandomState(0)
    a = 0.3 + 0.1 * rand.randn(n)
    b = 0.8 + 0.05 * rand.randn(n)
    mask = rand.rand(n) < 0.5
    samples = np.clip(a * mask + b * (1 - mask), 0.0, 1.0)
    return np.digitize(samples, np.linspace(0.0, 1.0, d)).astype('float32')

def generate_2d_data(n, dist):
    import itertools
    d1, d2 = dist.shape
    pairs = list(itertools.product(range(d1), range(d2)))
    idxs = np.random.choice(len(pairs), size=n, replace=True, p=dist.reshape(-1))
    samples = [pairs[i] for i in idxs]

    return np.array(samples).astype('float32')

# Cell
def train(model, train_loader, optimizer):
    model.train()
    for x in train_loader:
        loss = model.nll(x)
        optimizer.zero_grad()
        loss.backward()
        optimizer.step()


def eval_loss(model, data_loader):
    model.eval()
    total_loss = 0
    with torch.no_grad():
        for x in data_loader:
            loss = model.nll(x)
            total_loss += loss * x.shape[0]
        avg_loss = total_loss / len(data_loader.dataset)
    return avg_loss.item()


def train_epochs(model, train_loader, test_loader, train_args):
    epochs, lr = train_args['epochs'], train_args['lr']
    optimizer = optim.Adam(model.parameters(), lr=lr)

    train_losses, test_losses = [], []
    for epoch in range(epochs):
        model.train()

        train(model, train_loader, optimizer)
        train_loss = eval_loss(model, train_loader)
        train_losses.append(train_loss)

        if test_loader is not None:
            test_loss = eval_loss(model, test_loader)
            test_losses.append(test_loss)

        if epoch in [0, 2, 10, 50, 99]:
            plot_hist(train_loader.dataset.array, bins=d,
                      title=f'Epoch {epoch}', density=model.get_density())
    if test_loader is not None:
      print('Test Loss', test_loss)

    plot_train_curves(epochs, train_losses, test_losses, title='Training Curve')

# Cell
class Histogram(nn.Module):
    def __init__(self, d):
        super().__init__()
        self.d = d
        self.logits = nn.Parameter(torch.zeros(d), requires_grad=True)

    # Compute loss as negative log-likelihood
    def nll(self, x):
        logits = self.logits.unsqueeze(0).repeat(x.shape[0], 1) # batch_size x d
        return F.cross_entropy(logits, x.long())

    def get_density(self):
        x = np.linspace(-0.5, self.d - 0.5, 1000)
        with torch.no_grad():
            y = F.softmax(self.logits, dim=0).unsqueeze(1).repeat(1, 1000 // self.d).view(-1).numpy()
        return x, y

class MixtureOfLogistics(nn.Module):
    def __init__(self, d, n_mix=4):
        super().__init__()
        self.d = d
        self.n_mix = n_mix

        self.logits = nn.Parameter(torch.zeros(n_mix), requires_grad=True)
        self.means = nn.Parameter(torch.arange(n_mix).float() / (n_mix - 1) * d, requires_grad=True)
        self.log_scales = nn.Parameter(torch.randn(n_mix), requires_grad=True)

    def nll(self, x):
        x = x.unsqueeze(1).repeat(1, self.n_mix) # b x n_mix
        means, log_scales = self.means.unsqueeze(0), self.log_scales.unsqueeze(0) # 1 x n_mix
        inv_scales = torch.exp(-log_scales)

        plus_in = inv_scales * (x + 0.5 - means)
        min_in = inv_scales * (x - 0.5 - means)

        cdf_plus = torch.sigmoid(plus_in) # CDF of logistics at x + 0.5
        cdf_min = torch.sigmoid(min_in) # CDF of logistics at x - 0.5

        cdf_delta = cdf_plus - cdf_min # probability of x in bin [x - 0.5, x + 0.5]
        log_cdf_delta = torch.log(torch.clamp(cdf_delta, min=1e-12))
        log_cdf_plus = torch.sigmoid(inv_scales * (0.5 - means))
        log_cdf_min = torch.sigmoid(inv_scales * (d - 1.5 - means))

        x_log_probs = torch.where(x < 0.001, log_cdf_plus,
                                  torch.where(x > d - 1 - 1e-3,
                                              log_cdf_min, log_cdf_delta))
        pi_log_probs = F.log_softmax(self.logits, dim=0).unsqueeze(0)
        log_probs = x_log_probs + pi_log_probs
        return -torch.mean(torch.logsumexp(log_probs, dim=1))

    def get_density(self):
        x = np.linspace(-0.5, self.d - 1 + 0.5, 1000)

        with torch.no_grad():
            x_pt = torch.FloatTensor(x).unsqueeze(1)
            means, log_scales = self.means.unsqueeze(0), self.log_scales.unsqueeze(0)
            pi_log_probs = F.log_softmax(self.logits, dim=0).unsqueeze(0)

            # Calculate pdf of logistic distributions and weight them
            # according to learned mixture probabilities
            x_in = (x_pt - means) * torch.exp(-log_scales)
            log_pdf = x_in - log_scales - 2 * F.softplus(x_in)
            log_pdf = log_pdf + pi_log_probs
            log_pdf = torch.logsumexp(log_pdf, dim=1)
            pdf = log_pdf.exp()

        return x, pdf.numpy()


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/custom_tf_training.ipynb (unless otherwise specified).

__all__ = []

# Cell
import os
from nbdev.showdoc import *
from fastcore.test import *


import tensorflow as tf
import sklearn
import numpy as np
from sklearn.datasets import fetch_california_housing
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import StandardScaler
import matplotlib as mpl
import matplotlib.pyplot as plt
mpl.rc('axes', labelsize=14)
mpl.rc('xtick', labelsize=12)
mpl.rc('ytick', labelsize=12)


test_eq(sklearn.__version__ > "0.20", True)
test_eq(tf.__version__ > "2.0.0", True)


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/function_approximation.ipynb (unless otherwise specified).

__all__ = ['KerasPipeline', 'HighLevelKerasPipeline', 'HighLevelKerasPipelineMultiTarget']

# Cell
from abc import ABC, abstractmethod
from fastcore.test import *
import jovsatools
from jovsatools import data_generator
from nbdev.showdoc import *
import numpy as np
import tensorflow as tf
from tensorflow import keras
from tensorflow.keras.layers.experimental.preprocessing import Rescaling

test_eq(tf.__version__>= "2.2.0", True)

# Cell

class KerasPipeline(ABC):
    """Scafolding for a tf.keras model building pipeline.

    This base class will contain scafolding for all pipelines
    used. The goal of this base class is mainly to add safeguard
    and assumption enforcement.
    """
    def __init__(self):
        # safe guard: to ensure that various session don't collide
        keras.backend.clear_session()
        self.train_x, self.train_y, self.test_x, self.test_y = None, None, None, None
        self.model = None
        self.evaluation = None
        self.train_history = None

    @abstractmethod
    def get_data(self):
        pass

    @abstractmethod
    def build_model(self):
        pass

    @abstractmethod
    def run_training(self):
        pass

    @abstractmethod
    def evaluate_pipeline(self):
        pass

    def plot_model_shapes(self, show_shapes=True, show_layer_names=True):
        return keras.utils.plot_model(self.model,
                                      show_shapes=show_shapes,
                                      show_layer_names=show_layer_names)

    def keras_model_summary(self):
        return self.model.summary()

    def __call__(self, verbose=0):
        ########################################
        # step 1: get data
        ########################################
        self.get_data()
        # checking that self.get_data() is implimented as expected
        assert list(
            map(lambda x: x is not None, [self.train_x, self.train_y, self.test_x, self.test_y])
            )  == [True] * 4

        ########################################
        # step 2: build model
        ########################################
        self.build_model()
        # checking that self.model() is implimented as expected
        assert self.model is not None

        ########################################
        # step 3: train model
        ########################################
        print("\n", "=-="*10, "MODEL TRAINING", "=-="*10, "\n")
        self.run_training(verbose)
        # checking that self.train() is implimented as expected
        assert self.train_history is not None
        assert self.model._is_compiled == True

        if verbose > 0:
            print("\n", "=-="*10, "MODEL SUMMARY", "=-="*10, "\n")
            self.keras_model_summary()

        ########################################
        # step 4: evaluate pipeline
        ########################################
        print("\n", "=-="*10, "PIPELINE EVALUATION", "=-="*10, "\n")
        self.evaluate_pipeline()
        # checking that self.evaluate() is implimented as expected
        assert self.evaluation is not None

        # TODO (jovsa): run tests that ensure pipeline trains model above a benchmark

# Cell
class HighLevelKerasPipeline(KerasPipeline):
    def __init__(self, sample_n, additional_y):
        self.sample_n = sample_n
        self.additional_y = additional_y
        super().__init__()

    def get_data(self):
        data = data_generator.MNISTDataGenerator(self.additional_y)
        datasets  = data(self.sample_n)
        self.train_x, self.train_y, self.test_x, self.test_y = datasets

    def build_model(self):
        inputs = keras.Input(shape=(784, 1), name="inputs")
        # Rescale images to [0, 1]
        x = Rescaling(scale=1./255)(inputs)
        x = keras.layers.Flatten()(x)
        x = keras.layers.Dense(units=64, activation='relu')(x)
        output = keras.layers.Dense(units=10, activation='softmax', name="output")(x)

        self.model = keras.Model(inputs=inputs, outputs=output)

    def run_training(self, verbose):
        self.model.compile(
            optimizer=keras.optimizers.RMSprop(learning_rate=1e-3),
            loss=keras.losses.SparseCategoricalCrossentropy(),
            metrics=[keras.metrics.SparseCategoricalAccuracy()])

        self.train_history = self.model.fit(
            x=self.train_x, y=self.train_y, epochs=10, batch_size=64,
            verbose=verbose, validation_split=0.1)

    def evaluate_pipeline(self):
        self.evaluation = self.model.evaluate(x=self.test_x, y=self.test_y)

# Cell
class HighLevelKerasPipelineMultiTarget(KerasPipeline):
    def __init__(self, sample_n, additional_y):
        self.sample_n = sample_n
        self.additional_y = additional_y
        super().__init__()

    def get_data(self):
        data = data_generator.MNISTDataGenerator(self.additional_y)
        datasets  = data(self.sample_n)
        self.train_x, self.train_y, self.test_x, self.test_y = datasets


    def build_model(self):
        inputs = keras.Input(shape=(784, 1), name="inputs")
        # Rescale images to [0, 1]
        x = Rescaling(scale=1./255)(inputs)
        x = keras.layers.Flatten()(x)
        x = keras.layers.Dense(units=64, activation='relu')(x)
        output_1 = keras.layers.Dense(units=10, activation='softmax', name="output_0")(x)
        output_2 = keras.layers.Dense(units=1, activation='sigmoid', name="output_1")(x)

        self.model = keras.Model(inputs=inputs, outputs=[output_1, output_2])

    def custom_loss_0(self):
        def loss_fn(y_true, y_pred):
            l = keras.losses.SparseCategoricalCrossentropy()
            return l(y_true, y_pred)
        return loss_fn

    def custom_loss_1(self):
        def loss_fn(y_true, y_pred):
            l = keras.losses.CategoricalCrossentropy()
            return l(y_true, y_pred)
        return loss_fn


    def run_training(self, verbose):

        self.model.compile(
            optimizer=keras.optimizers.RMSprop(learning_rate=1e-3),
            loss = {
                "output_0": self.custom_loss_0(),
                "output_1": self.custom_loss_1(),
            },
            metrics={
                "output_0": keras.metrics.SparseCategoricalAccuracy(),
                "output_1": "categorical_accuracy"
            })

        self.train_history = self.model.fit(
            x=self.train_x,
            y={
                "output_0": self.train_y[:,0],
                "output_1": self.train_y[:,1]
            },
            epochs=30, batch_size=64,
            verbose=verbose, validation_split=0.1)

    def evaluate_pipeline(self):
        self.evaluation = self.model.evaluate(x=self.test_x, y={"output_0": self.test_y[:,0], "output_1": self.test_y[:,1]})


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/dice_experiments.ipynb (unless otherwise specified).

__all__ = []

# Cell
from nbdev.showdoc import *
import jovsatools
import fastcore
import numpy as np


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/gdrive_activity_tracking.ipynb (unless otherwise specified).

__all__ = ['truncated', 'getOneOf', 'getTimeInfo', 'getActionInfo', 'getUserInfo', 'getActorInfo', 'getTargetInfo',
           'initialize_data', 'get_authors', 'convert_dates', 'get_drive_activity', 'SCOPES', 'identify_actor',
           'check_data', 'clean_data', 'get_data']

# Cell
from nbdev.showdoc import *
import jovsatools
import fastcore
import pickle
import os.path
from googleapiclient.discovery import build
from google_auth_oauthlib.flow import InstalledAppFlow
from google.auth.transport.requests import Request
from collections import defaultdict
import pandas as pd
import numpy as np
import datetime
import seaborn as sns

# Cell
# Returns a string representation of the first elements in a list.
def truncated(array):
    array = list(array)
    contents = ', '.join(array[:])
    return u'[{0}]'.format(contents)


# Returns the name of a set property in an object, or else "unknown".
def getOneOf(obj):
    for key in obj:
        return key
    return 'unknown'


# Returns a time associated with an activity.
def getTimeInfo(activity):
    if 'timestamp' in activity:
        return activity['timestamp']
    if 'timeRange' in activity:
        return activity['timeRange']['endTime']
    return 'unknown'


# Returns the type of action.
def getActionInfo(actionDetail):
    return getOneOf(actionDetail)


# Returns user information, or the type of user if not a known user.
def getUserInfo(user):
    if 'knownUser' in user:
        knownUser = user['knownUser']
        isMe = knownUser.get('isCurrentUser', False)
        return knownUser['personName']
    return getOneOf(user)


# Returns actor information, or the type of actor if not a user.
def getActorInfo(actor):
    if 'user' in actor:
        return getUserInfo(actor['user'])
    return getOneOf(actor)


# Returns the type of a target and an associated title.
def getTargetInfo(target):
    if 'driveItem' in target:
        title = target['driveItem'].get('title', 'unknown')
        return 'driveItem:"{0}"'.format(title)
    if 'drive' in target:
        title = target['drive'].get('title', 'unknown')
        return 'drive:"{0}"'.format(title)
    if 'fileComment' in target:
        parent = target['fileComment'].get('parent', {})
        title = parent.get('title', 'unknown')
        return 'fileComment:"{0}"'.format(title)
    return '{0}:unknown'.format(getOneOf(target))

def initialize_data(feature_list):
    data = {}
    for feature in feature_list:
        data[feature] = []
    return data

def get_authors(df, actor_column):
    authors = set()
    for actors in df[actor_column].values:
        for actor in actors:
            authors.add(actor)
    return authors

def convert_dates(x):
    res = None
    for fmt in ('%Y-%m-%dT%H:%M:%S.%fZ', '%Y-%m-%dT%H:%M:%SZ'):
        try:
            res = datetime.datetime.strptime(x, fmt)
        except ValueError:
            pass
    assert res
    return res

# Cell

# If modifying these scopes, delete the file token.pickle.
SCOPES = ['https://www.googleapis.com/auth/drive.activity.readonly']


def get_drive_activity(page_size, loc, debug=False):
    """Shows basic usage of the Drive Activity API.

    Prints information about the last 10 events that occured the user's Drive.
    """
    creds = None
    # The file token.pickle stores the user's access and refresh tokens, and is
    # created automatically when the authorization flow completes for the first
    # time.
    if os.path.exists('token.pickle'):
        with open('token.pickle', 'rb') as token:
            creds = pickle.load(token)
    # If there are no (valid) credentials available, let the user log in.
    if not creds or not creds.valid:
        if creds and creds.expired and creds.refresh_token:
            creds.refresh(Request())
        else:
            flow = InstalledAppFlow.from_client_secrets_file(
                'credentials.json', SCOPES)
            creds = flow.run_local_server(port=0)
        # Save the credentials for the next run
        with open('token.pickle', 'wb') as token:
            pickle.dump(creds, token)

    service = build('driveactivity', 'v2', credentials=creds)

    # Call the Drive Activity API
    results = service.activity().query(body={
        'pageSize': page_size,
        'ancestorName': 'items/{}'.format(loc)
    }).execute()
    activities = results.get('activities', [])

    if debug:
        if not activities:
            print('No activity.')
        else:
            print('Recent activity:')
            for activity in activities:
                time = getTimeInfo(activity)
                action = getActionInfo(activity['primaryActionDetail'])
                actors = map(getActorInfo, activity['actors'])
                targets = map(getTargetInfo, activity['targets'])
                print(u'{0}: {1}, {2}, {3}'.format(time,
                                                   truncated(actors),
                                                   action,
                                                   truncated(targets)
                                                  ))

    feature_list = ['time', 'action', 'actor', 'target']
    data = initialize_data(feature_list)
    for activity in activities:
            time = getTimeInfo(activity)
            action = getActionInfo(activity['primaryActionDetail'])
            actors = list(map(getActorInfo, activity['actors']))
            targets = list(map(getTargetInfo, activity['targets']))

            data['time'].append(time)
            data['action'].append(action)
            data['actor'].append(actors)
            data['target'].append(targets)
    return pd.DataFrame.from_dict(data)

# Cell

def identify_actor(actor, actor_names):
    assert isinstance(actor, list)
    if len(actor) == 1:
        actor = actor[0].strip().split("/")[-1]
        return actor_names.get(actor, "UNOWN ACTOR")
    else:
        return "TOO MANY ACTORS"

def check_data(data, checks, actor_names):

    assert checks
    assert set(data['actor_name']) == set(actor_names.values()), "unknown actors"
    assert checks['min_year'] in  set(data['year']), "missing some old year data"
    assert checks['min_month'] in  set(data['month']), "missing some old month data"


def clean_data(data, actor_names):
    # time
    time_cleaned = 'time_cleaned_UTC'
    data[time_cleaned] = data.apply(lambda x: convert_dates(x['time']), axis=1)
    data['hour'] = data[time_cleaned].dt.hour
    data['day'] = data[time_cleaned].dt.day
    data['day_of_week'] = data[time_cleaned].dt.dayofweek
    data['day_name'] = data[time_cleaned].dt.day_name()
    data['week'] = data[time_cleaned].dt.week
    data['month'] = data[time_cleaned].dt.month
    data['year'] = data[time_cleaned].dt.year


    # actor
    if actor_names:
        data['actor_name'] = data.apply(lambda x: identify_actor(x['actor'], actor_names), axis=1)
    return data

def get_data(page_size, loc, actor_names, checks):
    data = get_drive_activity(page_size, loc)
    data = clean_data(data, actor_names)

    # run checks
    check_data(data, checks, actor_names)
    return data


# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"generate_1d_data": "custom_pytorch_training.ipynb",
         "generate_2d_data": "custom_pytorch_training.ipynb",
         "train": "custom_pytorch_training.ipynb",
         "eval_loss": "custom_pytorch_training.ipynb",
         "train_epochs": "custom_pytorch_training.ipynb",
         "Histogram": "custom_pytorch_training.ipynb",
         "MixtureOfLogistics": "custom_pytorch_training.ipynb",
         "MNISTDataGenerator": "data_generator.ipynb",
         "KerasPipeline": "function_approximation.ipynb",
         "HighLevelKerasPipeline": "function_approximation.ipynb",
         "HighLevelKerasPipelineMultiTarget": "function_approximation.ipynb",
         "truncated": "gdrive_activity_tracking.ipynb",
         "getOneOf": "gdrive_activity_tracking.ipynb",
         "getTimeInfo": "gdrive_activity_tracking.ipynb",
         "getActionInfo": "gdrive_activity_tracking.ipynb",
         "getUserInfo": "gdrive_activity_tracking.ipynb",
         "getActorInfo": "gdrive_activity_tracking.ipynb",
         "getTargetInfo": "gdrive_activity_tracking.ipynb",
         "initialize_data": "gdrive_activity_tracking.ipynb",
         "get_authors": "gdrive_activity_tracking.ipynb",
         "convert_dates": "gdrive_activity_tracking.ipynb",
         "get_drive_activity": "gdrive_activity_tracking.ipynb",
         "SCOPES": "gdrive_activity_tracking.ipynb",
         "identify_actor": "gdrive_activity_tracking.ipynb",
         "check_data": "gdrive_activity_tracking.ipynb",
         "clean_data": "gdrive_activity_tracking.ipynb",
         "get_data": "gdrive_activity_tracking.ipynb",
         "MoneyAgent": "sample_mesa_model.ipynb",
         "MoneyModel": "sample_mesa_model.ipynb"}

modules = ["custom_pytorch_training.py",
           "custom_tf_training.py",
           "customizing_tf_fit.py",
           "data_generator.py",
           "dice_experiments.py",
           "function_approximation.py",
           "gdrive_activity_tracking.py",
           "sample_mesa.py"]

doc_url = "https://jovsa.github.io/jovsatools/"

git_url = "https://github.com/jovsa/jovsatools/tree/master/"

def custom_doc_links(name): return None
